package com.xy.redis;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.support.SFunction;
import com.xy.collections.list.JArrayList;
import com.xy.collections.list.JList;
import com.xy.collections.map.JMap;
import com.xy.entity.*;
import com.xy.entity.join.SysMenuJoinSysRoleMenu;
import com.xy.mapper.SysMenuMapper;
import com.xy.mapper.SysRoleMapper;
import com.xy.mapper.SysSystemMapper;
import com.xy.utils.Beans;
import com.xy.utils.Emptys;
import com.xy.utils.RedisService;
import com.xy.utils.SpringBeanUtils;
import icu.mhb.mybatisplus.plugln.core.JoinLambdaWrapper;
import lombok.Data;
import lombok.experimental.Accessors;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 权限系统redis工具类
 */
@Component
public class RedisCache {

    /**
     * 权限系统默认系统ID，代表本系统
     */
    private static final String DEFAULT_STSTEM_ID = "1";

    private static RedisService<List<CacheEntity>> getRedisService() {
        return SpringBeanUtils.getBean(RedisService.class);
    }

    private static String getKey(Long loginId) {
        return String.format("%s:%d", "menu", loginId);
    }

    /**
     * 缓存权限相关到redis
     *
     * @param loginId 登录id
     */
    public static void cache(Long loginId) {
        SysSystemMapper sysSystemMapper = SpringBeanUtils.getBean(SysSystemMapper.class);
        SysRoleMapper sysRoleMapper = SpringBeanUtils.getBean(SysRoleMapper.class);
        SysMenuMapper sysMenuMapper = SpringBeanUtils.getBean(SysMenuMapper.class);
        RedisService<List<CacheEntity>> roleMenuRedisService = getRedisService();
        //获取登录人所有系统
        List<SysSystem> sysSystems = sysSystemMapper.selectList(new LambdaQueryWrapper<SysSystem>()
                .eq(SysSystem::getSysUserId, loginId)
                .eq(SysSystem::getStatus, true)
        );
        if (!Emptys.check(sysSystems)) {
            return;
        }
        //获取登录人所有角色
        JoinLambdaWrapper<SysRole> sysRoleJoinLambdaWrapper = new JoinLambdaWrapper<>(SysRole.class)
                .eq(SysRole::getStatus, true)
                .innerJoin(SysSystemUserRole.class, SysSystemUserRole::getRoleId, SysRole::getId)
                .eq(SysSystemUserRole::getSystemUserId, loginId)
                .end();
        List<SysRole> sysRoles = sysRoleMapper.joinSelectList(sysRoleJoinLambdaWrapper, SysRole.class);
        JList<Long> sysRoleIds = new JArrayList<>(sysRoles).getProperty(SysRole::getId);
        if (sysRoleIds.size() == 0) {
            return;
        }
        //获取登录人所有角色菜单
        JoinLambdaWrapper<SysMenu> sysMenuJoinLambdaWrapper = new JoinLambdaWrapper<>(SysMenu.class)
                .eq(SysMenu::getStatus, true)
                .innerJoin(SysRoleMenu.class, SysRoleMenu::getMenuId, SysMenu::getId)
                .selectAs(cb -> cb
                        .add(SysRoleMenu::getSelectIsAdmin)
                        .add(SysRoleMenu::getRoleId)
                )
                .in(SysRoleMenu::getRoleId, sysRoleIds)
                .end();
        List<SysMenuJoinSysRoleMenu> sysMenus = sysMenuMapper.joinSelectList(sysMenuJoinLambdaWrapper, SysMenuJoinSysRoleMenu.class);
        JMap<Long, List<SysMenuJoinSysRoleMenu>> sysMenuMaps = new JArrayList<>(sysMenus).toMap(SysMenuJoinSysRoleMenu::getRoleId).group();
        sysSystems.forEach(sysSystem -> {
            List<CacheEntity> cacheEntitys = new ArrayList<>();
            sysRoles.forEach(sysRole -> {
                CacheEntity cacheEntity = new CacheEntity()
                        .setRoleId(sysRole.getId())
                        .setList(sysMenuMaps.get(sysRole.getId()));
                cacheEntitys.add(cacheEntity);
            });
            roleMenuRedisService.setMap(getKey(loginId), sysSystem.getId().toString(), cacheEntitys);
        });
    }

    /**
     * 获取菜单
     *
     * @param loginId 登录id
     * @return
     */
    public static List<SysMenuJoinSysRoleMenu> getMenu(Long loginId) {
        //获取redis缓存
        List<CacheEntity> cacheEntities = getRedisService().getMap(getKey(loginId), DEFAULT_STSTEM_ID);
        //角色菜单累加
        JList<SysMenuJoinSysRoleMenu> sysMenuJoinSysRoleMenus = new JArrayList<>();
        cacheEntities.forEach(cacheEntity -> sysMenuJoinSysRoleMenus.addAll(cacheEntity.getList()));
        //去重
        sysMenuJoinSysRoleMenus.comparing(SysMenuJoinSysRoleMenu::getId);
        return sysMenuJoinSysRoleMenus;
    }

    /**
     * 获取菜单树
     *
     * @param loginId 登录id
     * @return
     */
    public static List<SysMenuJoinSysRoleMenu> getMenuTree(Long loginId) {
        //获取菜单
        JList<SysMenuJoinSysRoleMenu> sysMenuJoinSysRoleMenus = new JArrayList<>(getMenu(loginId));
        //生成树
        JList<SysMenuJoinSysRoleMenu> tops = sysMenuJoinSysRoleMenus.filter().isNull(SysMenuJoinSysRoleMenu::getPaterId).list();
        for (SysMenuJoinSysRoleMenu top : tops) {
            List<SysMenuJoinSysRoleMenu> tree = Beans.tree(top, sysMenuJoinSysRoleMenus, SysMenuJoinSysRoleMenu::getId, SysMenuJoinSysRoleMenu::getPaterId, SysMenuJoinSysRoleMenu::getSonSysMenu);
            top.setSonSysMenu(tree);
        }
        return tops;
    }

    /**
     * 判断数据查询权限
     *
     * @param loginId 登录id
     * @return true=管理员数据查询权限 false=普通查询权限
     */
    public static boolean getDataAuth(Long loginId) {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        //获取redis缓存
        List<CacheEntity> cacheEntities = getRedisService().getMap(getKey(loginId), DEFAULT_STSTEM_ID);
        //遍历角色
        for (CacheEntity cacheEntity : cacheEntities) {
            //根据 接口uri和管理员数据查询权限=true 查找角色菜单，找到则有管理员数据查询权限，反之普通查询权限
            SysMenuJoinSysRoleMenu sysMenuJoinSysRoleMenu = new JArrayList<>(cacheEntity.getList()).filter()
                    .eq(SysMenuJoinSysRoleMenu::getInterfaceUri, request.getRequestURI())
                    .eq(SysMenuJoinSysRoleMenu::getSelectIsAdmin, true)
                    .object();
            if (sysMenuJoinSysRoleMenu != null) {
                return true;
            }
        }
        return false;
    }

    /**
     * 判断数据查询权限
     *
     * @param loginId            登录id
     * @param lambdaQueryWrapper 查询构造器
     * @param function           普通查询权限时，条件lambda引用字段
     * @return true=管理员数据查询权限 false=普通查询权限
     */
    public static <T> boolean getDataAuth(Long loginId, LambdaQueryWrapper<T> lambdaQueryWrapper, SFunction<T, ?> function) {
        boolean dataAuth = getDataAuth(loginId);
        if (!dataAuth) {
            lambdaQueryWrapper.eq(function, loginId);
        }
        return false;
    }

    /**
     * 获取所有系统ID
     *
     * @param loginId 登录id
     * @return
     */
    public static List<Long> getSystemIds(Long loginId) {
        Map<String, List<CacheEntity>> map = getRedisService().getMap(getKey(loginId));
        List<Long> list = new ArrayList<>(map.size());
        map.forEach((s, cacheEntities) -> list.add(Long.parseLong(s)));
        return list;
    }

    /**
     * 获取所有角色ID
     *
     * @param loginId 登录id
     * @return
     */
    public static List<Long> getRoleIds(Long loginId) {
        List<CacheEntity> cacheEntities = getRedisService().getMap(getKey(loginId), DEFAULT_STSTEM_ID);
        List<Long> list = new ArrayList<>(cacheEntities.size());
        cacheEntities.forEach(cacheEntity -> list.add(Long.parseLong(cacheEntity.getRoleId().toString())));
        return list;
    }

    @Data
    @Accessors(chain = true)
    public static class CacheEntity {

        /**
         * 角色id
         */
        private Long roleId;

        /**
         * 菜单集合
         */
        private List<SysMenuJoinSysRoleMenu> list;

    }

}
