package com.xy.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.xy.utils.PageBean;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import java.time.LocalDate;
import java.time.LocalDateTime;

@Data
@ApiModel("权限用户交互类")
@Accessors(chain = true)
public class SysSystemUserDto {

    @Data
    @ApiModel("查询交互类")
    @Accessors(chain = true)
    public static class Select extends Vo {
        @ApiModelProperty("分页对象")
        private PageBean page;

        @JsonFormat(pattern = "yyyy-MM-dd")
        @ApiModelProperty(value = "创建时间-起")
        private LocalDate beginCreateTime;

        @JsonFormat(pattern = "yyyy-MM-dd")
        @ApiModelProperty(value = "创建时间-始")
        private LocalDate endCreateTime;

        public LocalDateTime getBeginCreateTime() {
            return beginCreateTime == null ? null : beginCreateTime.atTime(0, 0, 0);
        }

        public LocalDateTime getEndCreateTime() {
            return endCreateTime == null ? null : endCreateTime.atTime(23, 59, 59);
        }
    }

    @Data
    @Accessors(chain = true)
    @ApiModel("修改交互类")
    public static class Update {

        @NotNull(message = "id不能为空")
        @ApiModelProperty("唯一标识")
        private Long id;

        @ApiModelProperty("密码")
        private String password;

        @ApiModelProperty("状态 true=启用 false=封禁")
        private Boolean status;

        @ApiModelProperty(value = "是否是登录", hidden = true)
        private Boolean isLogin;

        @ApiModelProperty(value = "是否是重置密码")
        private Boolean isAgainPwd;
    }

    @Data
    @Accessors(chain = true)
    @ApiModel("登录交互类")
    public static class Login {
        @ApiModelProperty(value = "账号", required = true)
        @NotBlank(message = "account不能为空")
        @Pattern(regexp = "^[a-zA-z0-9]{6,11}$", message = "account必须为数字或字母，长度6-11位之间")
        private String account;

        @ApiModelProperty(value = "密码", required = true)
        @NotBlank(message = "password不能为空")
        @Pattern(regexp = "^[a-zA-z0-9]{6,11}$", message = "password必须为数字或字母，长度6-11位之间")
        private String password;
    }

    @Data
    @Accessors(chain = true)
    @ApiModel("返参类")
    public static class Vo {
        @ApiModelProperty("账号")
        private String account;

        @ApiModelProperty("最后登录时间")
        private LocalDateTime lastLoginTime;

        @ApiModelProperty("登录次数")
        private Integer loginNum;

        @ApiModelProperty("状态 true=启用 false=封禁")
        private Boolean status;

        @ApiModelProperty("唯一标识")
        private Long id;

        @ApiModelProperty("创建时间")
        private LocalDateTime createTime;

        @ApiModelProperty("更新时间")
        private LocalDateTime updateTime;

        @ApiModelProperty("创建人")
        private Long createUser;

        @ApiModelProperty("更新人")
        private Long updateUser;
    }
}
